"use strict";
/*!
 * Copyright (c) 2017-2019 Cliqz GmbH. All rights reserved.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
exports.__esModule = true;
var tslib_1 = require("tslib");
var adblocker_1 = require("@cliqz/adblocker");
var electron_1 = require("electron");
var path_1 = require("path");
var tldts_experimental_1 = require("tldts-experimental");
var PRELOAD_PATH = path_1.join(__dirname, './preload.js');
// https://stackoverflow.com/questions/48854265/why-do-i-see-an-electron-security-warning-after-updating-my-electron-project-t
process.env.ELECTRON_DISABLE_SECURITY_WARNINGS = 'true';
/**
 * Create an instance of `Request` from `Electron.OnBeforeRequestDetails`.
 */
function fromElectronDetails(_a) {
    var id = _a.id, url = _a.url, resourceType = _a.resourceType, referrer = _a.referrer, webContentsId = _a.webContentsId;
    return adblocker_1.Request.fromRawDetails({
        requestId: "" + id,
        sourceUrl: referrer,
        tabId: webContentsId,
        type: (resourceType || 'other'),
        url: url
    });
}
exports.fromElectronDetails = fromElectronDetails;
/**
 * Wrap `FiltersEngine` into a Electron-friendly helper class.
 */
var ElectronBlocker = /** @class */ (function (_super) {
    tslib_1.__extends(ElectronBlocker, _super);
    function ElectronBlocker() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onIsMutationObserverEnabled = function (event) {
            event.returnValue = _this.config.enableMutationObserver;
        };
        _this.onGetCosmeticFilters = function (event, url, msg) {
            // Extract hostname from sender's URL
            var parsed = tldts_experimental_1.parse(url);
            var hostname = parsed.hostname || '';
            var domain = parsed.domain || '';
            var _a = _this.getCosmeticsFilters({
                domain: domain,
                hostname: hostname,
                url: url,
                classes: msg.classes,
                hrefs: msg.hrefs,
                ids: msg.ids,
                // This needs to be done only once per frame
                getBaseRules: msg.lifecycle === 'start',
                getInjectionRules: msg.lifecycle === 'start',
                getRulesFromHostname: msg.lifecycle === 'start',
                // This will be done every time we get information about DOM mutation
                getRulesFromDOM: msg.lifecycle === 'dom-update'
            }), active = _a.active, styles = _a.styles, scripts = _a.scripts;
            if (active === false) {
                return;
            }
            // Inject custom stylesheets
            _this.injectStyles(event.sender, styles);
            // Inject scripts from content script
            event.sender.send('get-cosmetic-filters-response', {
                active: active,
                extended: [],
                scripts: scripts,
                styles: ''
            });
        };
        _this.onHeadersReceived = function (details, callback) {
            var e_1, _a;
            var CSP_HEADER_NAME = 'content-security-policy';
            var policies = [];
            var responseHeaders = (details.responseHeaders || {});
            if (details.resourceType === 'mainFrame' || details.resourceType === 'subFrame') {
                var rawCSP = _this.getCSPDirectives(fromElectronDetails(details));
                if (rawCSP !== undefined) {
                    policies.push.apply(policies, tslib_1.__spread(rawCSP.split(';').map(function (csp) { return csp.trim(); })));
                    try {
                        // Collect existing CSP headers from response
                        for (var _b = tslib_1.__values(Object.entries(responseHeaders)), _c = _b.next(); !_c.done; _c = _b.next()) {
                            var _d = tslib_1.__read(_c.value, 2), name_1 = _d[0], value = _d[1];
                            if (name_1.toLowerCase() === CSP_HEADER_NAME) {
                                if (Array.isArray(value)) {
                                    policies.push.apply(policies, tslib_1.__spread(value));
                                }
                                else {
                                    policies.push(value);
                                }
                                delete responseHeaders[name_1];
                            }
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                    responseHeaders[CSP_HEADER_NAME] = policies.join(';');
                    // @ts-ignore
                    callback({ responseHeaders: responseHeaders });
                    return;
                }
            }
            callback({});
        };
        _this.onBeforeRequest = function (details, callback) {
            var request = fromElectronDetails(details);
            if (request.isMainFrame()) {
                callback({});
                return;
            }
            var _a = _this.match(request), redirect = _a.redirect, match = _a.match;
            if (redirect) {
                callback({ redirectURL: redirect.dataUrl });
            }
            else if (match) {
                callback({ cancel: true });
            }
            else {
                callback({});
            }
        };
        return _this;
    }
    ElectronBlocker.prototype.enableBlockingInSession = function (ses) {
        if (this.config.loadNetworkFilters === true) {
            ses.webRequest.onHeadersReceived({ urls: ['<all_urls>'] }, this.onHeadersReceived);
            ses.webRequest.onBeforeRequest({ urls: ['<all_urls>'] }, this.onBeforeRequest);
        }
        if (this.config.loadCosmeticFilters === true) {
            electron_1.ipcMain.on('get-cosmetic-filters', this.onGetCosmeticFilters);
            electron_1.ipcMain.on('is-mutation-observer-enabled', this.onIsMutationObserverEnabled);
            ses.setPreloads(ses.getPreloads().concat([PRELOAD_PATH]));
        }
    };
    ElectronBlocker.prototype.disableBlockingInSession = function (ses) {
        if (this.config.loadNetworkFilters === true) {
            // NOTE - there is currently no support in Electron for multiple
            // webRequest listeners registered for the same event. This means that
            // adblocker's listeners can be overriden by other ones in the same
            // application (or that the adblocker can override another listener
            // registered previously). Because of this, the only way to disable the
            // adblocker is to remove all listeners for the events we are interested
            // in. In the future, we should consider implementing a webRequest
            // pipeline allowing to register multiple listeners for the same event.
            ses.webRequest.onHeadersReceived(null);
            ses.webRequest.onBeforeRequest(null);
        }
        if (this.config.loadCosmeticFilters === true) {
            ses.setPreloads(ses.getPreloads().filter(function (p) { return p !== PRELOAD_PATH; }));
            // @ts-ignore
            electron_1.ipcMain.removeListener('get-cosmetic-filters', this.onGetCosmeticFilters);
            // @ts-ignore
            electron_1.ipcMain.removeListener('is-mutation-observer-enabled', this.onIsMutationObserverEnabled);
        }
    };
    ElectronBlocker.prototype.injectStyles = function (sender, styles) {
        if (styles.length > 0) {
            sender.insertCSS(styles, {
                cssOrigin: 'user'
            });
        }
    };
    return ElectronBlocker;
}(adblocker_1.FiltersEngine));
exports.ElectronBlocker = ElectronBlocker;
// re-export @cliqz/adblocker symbols for convenience
tslib_1.__exportStar(require("@cliqz/adblocker"), exports);
//# sourceMappingURL=adblocker.js.map