"use strict";
/*!
 * Copyright (c) 2017-present Cliqz GmbH. All rights reserved.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
exports.__esModule = true;
var tslib_1 = require("tslib");
/**
 * Built-in fetch helpers can be used to initialize the adblocker from
 * pre-built presets or raw lists (fetched from multiple sources). In case of
 * failure (e.g. timeout), the whole process of initialization fails. Timeouts
 * are not so uncommon, and retrying to fetch usually succeeds.
 */
function fetchWithRetry(fetch, url) {
    var retry = 3;
    // Wrap `fetch` into a lightweight retry function which makes sure that if
    // fetching fails, it can be retried up to three times. Failure can happen if
    // the remote server times-out, but retrying fetching of the same URL will
    // usually succeed.
    var fetchWrapper = function () {
        return fetch(url)["catch"](function (ex) {
            if (retry > 0) {
                retry -= 1;
                return new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        fetchWrapper()
                            .then(resolve)["catch"](reject);
                    }, 500);
                });
            }
            throw ex;
        });
    };
    return fetchWrapper();
}
exports.fetchWithRetry = fetchWithRetry;
function fetchResource(fetch, url) {
    return fetchWithRetry(fetch, url).then(function (response) { return response.text(); });
}
function fetchPrebuilt(fetch, configUrl, engineVersion) {
    return fetchWithRetry(fetch, configUrl)
        .then(function (response) { return response.json(); })
        .then(function (allowedLists) { return fetchWithRetry(fetch, allowedLists.engines[engineVersion].url); })
        .then(function (response) { return response.arrayBuffer(); })
        .then(function (buffer) { return new Uint8Array(buffer); });
}
exports.fetchPrebuilt = fetchPrebuilt;
exports.adsLists = [
    'https://easylist.to/easylist/easylist.txt',
    'https://pgl.yoyo.org/adservers/serverlist.php?hostformat=adblockplus&showintro=0&mimetype=plaintext',
    'https://raw.githubusercontent.com/uBlockOrigin/uAssets/master/filters/annoyances.txt',
    'https://raw.githubusercontent.com/uBlockOrigin/uAssets/master/filters/badware.txt',
    'https://raw.githubusercontent.com/uBlockOrigin/uAssets/master/filters/filters.txt',
    'https://raw.githubusercontent.com/uBlockOrigin/uAssets/master/filters/resource-abuse.txt',
    'https://raw.githubusercontent.com/uBlockOrigin/uAssets/master/filters/unbreak.txt',
    'https://easylist.to/easylistgermany/easylistgermany.txt',
];
exports.adsAndTrackingLists = tslib_1.__spread(exports.adsLists, [
    'https://easylist.to/easylist/easyprivacy.txt',
    'https://raw.githubusercontent.com/uBlockOrigin/uAssets/master/filters/privacy.txt',
]);
exports.fullLists = tslib_1.__spread(exports.adsAndTrackingLists, [
    'https://easylist-downloads.adblockplus.org/fanboy-annoyance.txt',
    'https://easylist-downloads.adblockplus.org/easylist-cookie.txt',
]);
/**
 * Fetch latest version of enabledByDefault blocking lists.
 */
function fetchLists(fetch, urls) {
    return Promise.all(urls.map(function (url) { return fetchResource(fetch, url); }));
}
exports.fetchLists = fetchLists;
function getResourcesUrl(fetch) {
    return fetchWithRetry(fetch, 'https://cdn.cliqz.com/adblocker/resources/ublock-resources/metadata.json')
        .then(function (response) { return response.json(); })
        .then(function (_a) {
        var revisions = _a.revisions;
        return "https://cdn.cliqz.com/adblocker/resources/ublock-resources/" + revisions[revisions.length - 1] + "/list.txt";
    });
}
/**
 * Fetch latest version of uBlock Origin's resources, used to inject scripts in
 * the page or redirect request to data URLs.
 */
function fetchResources(fetch) {
    return getResourcesUrl(fetch).then(function (url) { return fetchResource(fetch, url); });
}
exports.fetchResources = fetchResources;
//# sourceMappingURL=fetch.js.map