/*!
 * Copyright (c) 2017-present Cliqz GmbH. All rights reserved.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
import { __extends, __read, __values } from "tslib";
import Config from '../config';
import StaticDataView from '../data-view';
import { EventEmitter } from '../events';
import { adsAndTrackingLists, adsLists, fetchLists, fetchPrebuilt, fetchResources, fullLists, } from '../fetch';
import { parseFilters } from '../lists';
import Request from '../request';
import Resources from '../resources';
import CosmeticFilterBucket from './bucket/cosmetic';
import NetworkFilterBucket from './bucket/network';
export var ENGINE_VERSION = 44;
// Polyfill for `btoa`
function btoaPolyfill(buffer) {
    if (typeof btoa !== 'undefined') {
        return btoa(buffer);
    }
    else if (typeof Buffer !== 'undefined') {
        return Buffer.from(buffer).toString('base64');
    }
    return buffer;
}
var FilterEngine = /** @class */ (function (_super) {
    __extends(FilterEngine, _super);
    function FilterEngine(_a) {
        var _b = _a === void 0 ? {} : _a, 
        // Optionally initialize the engine with filters
        _c = _b.cosmeticFilters, 
        // Optionally initialize the engine with filters
        cosmeticFilters = _c === void 0 ? [] : _c, _d = _b.networkFilters, networkFilters = _d === void 0 ? [] : _d, _e = _b.config, config = _e === void 0 ? new Config() : _e, _f = _b.lists, lists = _f === void 0 ? new Map() : _f;
        var _this = _super.call(this) || this;
        _this.config = new Config(config);
        // Subscription management: disabled by default
        _this.lists = lists;
        // $csp=
        _this.csp = new NetworkFilterBucket({ config: _this.config });
        // $generichide
        _this.genericHides = new NetworkFilterBucket({ config: _this.config });
        // @@filter
        _this.exceptions = new NetworkFilterBucket({ config: _this.config });
        // $important
        _this.importants = new NetworkFilterBucket({ config: _this.config });
        // $redirect
        _this.redirects = new NetworkFilterBucket({ config: _this.config });
        // All other filters
        _this.filters = new NetworkFilterBucket({ config: _this.config });
        // Cosmetic filters
        _this.cosmetics = new CosmeticFilterBucket({ config: _this.config });
        // Injections
        _this.resources = new Resources();
        if (networkFilters.length !== 0 || cosmeticFilters.length !== 0) {
            _this.update({
                newCosmeticFilters: cosmeticFilters,
                newNetworkFilters: networkFilters
            });
        }
        return _this;
    }
    /**
     * Create an instance of `FiltersEngine` (or subclass like `ElectronBlocker`,
     * etc.), from the list of subscriptions provided as argument (e.g.:
     * EasyList).
     *
     * Lists are fetched using the instance of `fetch` provided as a first
     * argument. Optionally resources.txt and config can be provided.
     */
    FilterEngine.fromLists = function (fetch, urls, config) {
        var _this = this;
        if (config === void 0) { config = {}; }
        var listsPromises = fetchLists(fetch, urls);
        var resourcesPromise = fetchResources(fetch);
        return Promise.all([listsPromises, resourcesPromise]).then(function (_a) {
            var _b = __read(_a, 2), lists = _b[0], resources = _b[1];
            var engine = _this.parse(lists.join('\n'), config);
            if (resources !== undefined) {
                engine.updateResources(resources, '' + resources.length);
            }
            return engine;
        });
    };
    /**
     * Initialize blocker of *ads only*.
     *
     * Attempt to initialize a blocking engine using a pre-built version served
     * from Cliqz's CDN. If this fails (e.g.: if no pre-built engine is available
     * for this version of the library), then falls-back to using `fromLists(...)`
     * method with the same subscriptions.
     */
    FilterEngine.fromPrebuiltAdsOnly = function (fetchImpl) {
        var _this = this;
        if (fetchImpl === void 0) { fetchImpl = fetch; }
        return fetchPrebuilt(fetchImpl, 'https://cdn.cliqz.com/adblocker/configs/desktop-ads/allowed-lists.json', ENGINE_VERSION)
            .then(function (buffer) { return _this.deserialize(buffer); })["catch"](function () {
            console.log('failed downloading pre-built, fallback to fetching lists');
            return _this.fromLists(fetchImpl, adsLists);
        });
    };
    /**
     * Same as `fromPrebuiltAdsOnly(...)` but also contains rules to block
     * tracking (i.e.: using extra lists such as EasyPrivacy and more).
     */
    FilterEngine.fromPrebuiltAdsAndTracking = function (fetchImpl) {
        var _this = this;
        if (fetchImpl === void 0) { fetchImpl = fetch; }
        return fetchPrebuilt(fetchImpl, 'https://cdn.cliqz.com/adblocker/configs/desktop-ads-trackers/allowed-lists.json', ENGINE_VERSION)
            .then(function (buffer) { return _this.deserialize(buffer); })["catch"](function () {
            console.log('failed downloading pre-built, fallback to fetching lists');
            return _this.fromLists(fetchImpl, adsAndTrackingLists);
        });
    };
    /**
     * Same as `fromPrebuiltAdsAndTracking(...)` but also contains annoyances
     * rules to block things like cookie notices.
     */
    FilterEngine.fromPrebuiltFull = function (fetchImpl) {
        var _this = this;
        if (fetchImpl === void 0) { fetchImpl = fetch; }
        return fetchPrebuilt(fetchImpl, 'https://cdn.cliqz.com/adblocker/configs/desktop-full/allowed-lists.json', ENGINE_VERSION)
            .then(function (buffer) { return _this.deserialize(buffer); })["catch"](function () {
            console.log('failed downloading pre-built, fallback to fetching lists');
            return _this.fromLists(fetchImpl, fullLists);
        });
    };
    FilterEngine.parse = function (filters, options) {
        if (options === void 0) { options = {}; }
        var config = new Config(options);
        return new this(Object.assign({}, parseFilters(filters, config), { config: config }));
    };
    FilterEngine.deserialize = function (serialized) {
        var buffer = StaticDataView.fromUint8Array(serialized, {
            enableCompression: false
        });
        // Before starting deserialization, we make sure that the version of the
        // serialized engine is the same as the current source code. If not, we
        // start fresh and create a new engine from the lists.
        var serializedEngineVersion = buffer.getUint8();
        if (ENGINE_VERSION !== serializedEngineVersion) {
            throw new Error("serialized engine version mismatch, expected " + ENGINE_VERSION + " but got " + serializedEngineVersion);
        }
        // Create a new engine with same options
        var config = Config.deserialize(buffer);
        // Optionally turn compression ON
        if (config.enableCompression) {
            buffer.enableCompression();
        }
        // Also make sure that the built-in checksum is correct. This allows to
        // detect data corruption and start fresh if the serialized version was
        // altered.
        if (config.integrityCheck) {
            var currentPos = buffer.pos;
            buffer.pos = serialized.length - 4;
            var checksum = buffer.checksum();
            var expected = buffer.getUint32();
            if (checksum !== expected) {
                throw new Error("serialized engine checksum mismatch, expected " + expected + " but got " + checksum);
            }
            buffer.pos = currentPos;
        }
        var engine = new this({ config: config });
        // Deserialize resources
        engine.resources = Resources.deserialize(buffer);
        // Deserialize lists
        var lists = new Map();
        var numberOfLists = buffer.getUint16();
        for (var i = 0; i < numberOfLists; i += 1) {
            lists.set(buffer.getASCII(), buffer.getASCII());
        }
        engine.lists = lists;
        // Deserialize buckets
        engine.importants = NetworkFilterBucket.deserialize(buffer, config);
        engine.redirects = NetworkFilterBucket.deserialize(buffer, config);
        engine.filters = NetworkFilterBucket.deserialize(buffer, config);
        engine.exceptions = NetworkFilterBucket.deserialize(buffer, config);
        engine.csp = NetworkFilterBucket.deserialize(buffer, config);
        engine.genericHides = NetworkFilterBucket.deserialize(buffer, config);
        engine.cosmetics = CosmeticFilterBucket.deserialize(buffer, config);
        return engine;
    };
    /**
     * Estimate the number of bytes needed to serialize this instance of
     * `FiltersEngine` using the `serialize(...)` method. It is used internally
     * by `serialize(...)` to allocate a buffer of the right size and you should
     * not have to call it yourself most of the time.
     *
     * There are cases where we cannot estimate statically the exact size of the
     * resulting buffer (due to alignement which need to be performed); this
     * method will return a safe estimate which will always be at least equal to
     * the real number of bytes needed, or bigger (usually of a few bytes only:
     * ~20 bytes is to be expected).
     */
    FilterEngine.prototype.getSerializedSize = function () {
        var e_1, _a;
        var estimatedSize = StaticDataView.sizeOfByte() + // engine version
            this.config.getSerializedSize() +
            this.resources.getSerializedSize() +
            this.filters.getSerializedSize() +
            this.exceptions.getSerializedSize() +
            this.importants.getSerializedSize() +
            this.redirects.getSerializedSize() +
            this.csp.getSerializedSize() +
            this.genericHides.getSerializedSize() +
            this.cosmetics.getSerializedSize() +
            4; // checksum
        try {
            // Estimate size of `this.lists` which stores information of checksum for each list.
            for (var _b = __values(this.lists), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 2), name_1 = _d[0], checksum = _d[1];
                estimatedSize += StaticDataView.sizeOfASCII(name_1) + StaticDataView.sizeOfASCII(checksum);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return estimatedSize;
    };
    /**
     * Creates a binary representation of the full engine. It can be stored
     * on-disk for faster loading of the adblocker. The `deserialize` static
     * method of Engine can be used to restore the engine.
     */
    FilterEngine.prototype.serialize = function (array) {
        var buffer = StaticDataView.fromUint8Array(array || new Uint8Array(this.getSerializedSize()), this.config);
        buffer.pushUint8(ENGINE_VERSION);
        // Config
        this.config.serialize(buffer);
        // Resources (js, resources)
        this.resources.serialize(buffer);
        // Serialize the state of lists (names and checksums)
        buffer.pushUint16(this.lists.size);
        var entries = Array.from(this.lists.entries()).sort();
        for (var i = 0; i < entries.length; i += 1) {
            buffer.pushASCII(entries[i][0]);
            buffer.pushASCII(entries[i][1]);
        }
        // Filters buckets
        this.importants.serialize(buffer);
        this.redirects.serialize(buffer);
        this.filters.serialize(buffer);
        this.exceptions.serialize(buffer);
        this.csp.serialize(buffer);
        this.genericHides.serialize(buffer);
        this.cosmetics.serialize(buffer);
        // Optionally append a checksum at the end
        if (this.config.integrityCheck) {
            buffer.pushUint32(buffer.checksum());
        }
        return buffer.subarray();
    };
    /**
     * Update engine with new filters or resources.
     */
    FilterEngine.prototype.loadedLists = function () {
        return Array.from(this.lists.keys());
    };
    FilterEngine.prototype.hasList = function (name, checksum) {
        return this.lists.has(name) && this.lists.get(name) === checksum;
    };
    /**
     * Update engine with `resources.txt` content.
     */
    FilterEngine.prototype.updateResources = function (data, checksum) {
        if (this.resources.checksum === checksum) {
            return false;
        }
        this.resources = Resources.parse(data, { checksum: checksum });
        return true;
    };
    FilterEngine.prototype.getFilters = function () {
        var cosmeticFilters = [];
        var networkFilters = [];
        return {
            cosmeticFilters: cosmeticFilters.concat(this.cosmetics.getFilters()),
            networkFilters: networkFilters.concat(this.filters.getFilters(), this.exceptions.getFilters(), this.importants.getFilters(), this.redirects.getFilters(), this.csp.getFilters(), this.genericHides.getFilters())
        };
    };
    /**
     * Update engine with new filters as well as optionally removed filters.
     */
    FilterEngine.prototype.update = function (_a) {
        var _b = _a.newNetworkFilters, newNetworkFilters = _b === void 0 ? [] : _b, _c = _a.newCosmeticFilters, newCosmeticFilters = _c === void 0 ? [] : _c, _d = _a.removedCosmeticFilters, removedCosmeticFilters = _d === void 0 ? [] : _d, _e = _a.removedNetworkFilters, removedNetworkFilters = _e === void 0 ? [] : _e;
        var updated = false;
        // Update cosmetic filters
        if (this.config.loadCosmeticFilters &&
            (newCosmeticFilters.length !== 0 || removedCosmeticFilters.length !== 0)) {
            updated = true;
            this.cosmetics.update(newCosmeticFilters, removedCosmeticFilters.length === 0 ? undefined : new Set(removedCosmeticFilters));
        }
        // Update network filters
        if (this.config.loadNetworkFilters &&
            (newNetworkFilters.length !== 0 || removedNetworkFilters.length !== 0)) {
            updated = true;
            var filters = [];
            var csp = [];
            var exceptions = [];
            var importants = [];
            var redirects = [];
            var genericHides = [];
            for (var i = 0; i < newNetworkFilters.length; i += 1) {
                var filter = newNetworkFilters[i];
                // NOTE: it's important to check for $generichide and $csp before
                // exceptions and important as we store all of them in the same filter
                // bucket. The check for exceptions is done at match-time directly.
                if (filter.isCSP()) {
                    csp.push(filter);
                }
                else if (filter.isGenericHide()) {
                    genericHides.push(filter);
                }
                else if (filter.isException()) {
                    exceptions.push(filter);
                }
                else if (filter.isImportant()) {
                    importants.push(filter);
                }
                else if (filter.isRedirect()) {
                    redirects.push(filter);
                }
                else {
                    filters.push(filter);
                }
            }
            var removedNetworkFiltersSet = removedNetworkFilters.length === 0 ? undefined : new Set(removedNetworkFilters);
            // Update buckets in-place
            this.importants.update(importants, removedNetworkFiltersSet);
            this.redirects.update(redirects, removedNetworkFiltersSet);
            this.filters.update(filters, removedNetworkFiltersSet);
            this.exceptions.update(exceptions, removedNetworkFiltersSet);
            this.csp.update(csp, removedNetworkFiltersSet);
            this.genericHides.update(genericHides, removedNetworkFiltersSet);
        }
        return updated;
    };
    FilterEngine.prototype.updateFromDiff = function (_a) {
        var added = _a.added, removed = _a.removed;
        var newCosmeticFilters = [];
        var newNetworkFilters = [];
        var removedCosmeticFilters = [];
        var removedNetworkFilters = [];
        if (removed !== undefined && removed.length !== 0) {
            var _b = parseFilters(removed.join('\n'), this.config), networkFilters = _b.networkFilters, cosmeticFilters = _b.cosmeticFilters;
            Array.prototype.push.apply(removedCosmeticFilters, cosmeticFilters);
            Array.prototype.push.apply(removedNetworkFilters, networkFilters);
        }
        if (added !== undefined && added.length !== 0) {
            var _c = parseFilters(added.join('\n'), this.config), networkFilters = _c.networkFilters, cosmeticFilters = _c.cosmeticFilters;
            Array.prototype.push.apply(newCosmeticFilters, cosmeticFilters);
            Array.prototype.push.apply(newNetworkFilters, networkFilters);
        }
        return this.update({
            newCosmeticFilters: newCosmeticFilters,
            newNetworkFilters: newNetworkFilters,
            removedCosmeticFilters: removedCosmeticFilters.map(function (f) { return f.getId(); }),
            removedNetworkFilters: removedNetworkFilters.map(function (f) { return f.getId(); })
        });
    };
    /**
     * Return a list of HTML filtering rules.
     */
    FilterEngine.prototype.getHtmlFilters = function (_a) {
        var 
        // Page information
        url = _a.url, hostname = _a.hostname, domain = _a.domain;
        var htmlSelectors = [];
        if (this.config.enableHtmlFiltering === false || this.config.loadCosmeticFilters === false) {
            return htmlSelectors;
        }
        var rules = this.cosmetics.getHtmlRules({
            domain: domain || '',
            hostname: hostname
        });
        for (var i = 0; i < rules.length; i += 1) {
            var rule = rules[i];
            var extended = rule.getExtendedSelector();
            if (extended !== undefined) {
                htmlSelectors.push(extended);
            }
        }
        if (htmlSelectors.length !== 0) {
            this.emit('html-filtered', htmlSelectors, url);
        }
        return htmlSelectors;
    };
    /**
     * Given `hostname` and `domain` of a page (or frame), return the list of
     * styles and scripts to inject in the page.
     */
    FilterEngine.prototype.getCosmeticsFilters = function (_a) {
        var 
        // Page information
        url = _a.url, hostname = _a.hostname, domain = _a.domain, 
        // DOM information
        classes = _a.classes, hrefs = _a.hrefs, ids = _a.ids, 
        // Allows to specify which rules to return
        _b = _a.getBaseRules, 
        // Allows to specify which rules to return
        getBaseRules = _b === void 0 ? true : _b, _c = _a.getInjectionRules, getInjectionRules = _c === void 0 ? true : _c, _d = _a.getRulesFromDOM, getRulesFromDOM = _d === void 0 ? true : _d, _e = _a.getRulesFromHostname, getRulesFromHostname = _e === void 0 ? true : _e;
        if (this.config.loadCosmeticFilters === false) {
            return {
                active: false,
                extended: [],
                scripts: [],
                styles: ''
            };
        }
        // Check if there is some generichide
        var genericHides = this.genericHides.matchAll(Request.fromRawDetails({
            domain: domain || '',
            hostname: hostname,
            url: url,
            sourceDomain: '',
            sourceHostname: '',
            sourceUrl: ''
        }));
        // Get $generichide filter with highest priority:
        // $generichide,important > $generichide > @@$generichide
        var genericHideFilter = null;
        var currentScore = 0;
        for (var i = 0; i < genericHides.length; i += 1) {
            var filter = genericHides[i];
            // To encode priority between filters, we create a bitmask with the following:
            // $important,generichide = 100 (takes precedence)
            // $generichide           = 010 (exception to @@$generichide)
            // @@$generichide         = 001 (forbids generic hide filters)
            var score = (filter.isImportant() ? 4 : 0) | (filter.isException() ? 1 : 2);
            // Highest `score` has precedence
            if (score > currentScore) {
                currentScore = score;
                genericHideFilter = filter;
            }
        }
        // Check that there is at least one $generichide match and no exception
        var allowGenericHides = genericHideFilter === null || genericHideFilter.isException() === false;
        // Lookup injections as well as stylesheets
        var _f = this.cosmetics.getCosmeticsFilters({
            domain: domain || '',
            hostname: hostname,
            classes: classes,
            hrefs: hrefs,
            ids: ids,
            allowGenericHides: allowGenericHides,
            getBaseRules: getBaseRules,
            getInjectionRules: getInjectionRules,
            getRulesFromDOM: getRulesFromDOM,
            getRulesFromHostname: getRulesFromHostname
        }), injections = _f.injections, stylesheet = _f.stylesheet;
        // Perform interpolation for injected scripts
        var scripts = [];
        for (var i = 0; i < injections.length; i += 1) {
            var script = injections[i].getScript(this.resources.js);
            if (script !== undefined) {
                this.emit('script-injected', script, url);
                scripts.push(script);
            }
        }
        // Emit events
        if (stylesheet.length !== 0) {
            this.emit('style-injected', stylesheet, url);
        }
        return {
            active: true,
            extended: [],
            scripts: scripts,
            styles: stylesheet
        };
    };
    /**
     * Given a `request`, return all matching network filters found in the engine.
     */
    FilterEngine.prototype.matchAll = function (request) {
        var filters = [];
        if (request.isSupported) {
            Array.prototype.push.apply(filters, this.importants.matchAll(request));
            Array.prototype.push.apply(filters, this.filters.matchAll(request));
            Array.prototype.push.apply(filters, this.exceptions.matchAll(request));
            Array.prototype.push.apply(filters, this.csp.matchAll(request));
            Array.prototype.push.apply(filters, this.genericHides.matchAll(request));
            Array.prototype.push.apply(filters, this.redirects.matchAll(request));
        }
        return new Set(filters);
    };
    /**
     * Given a "main_frame" request, check if some content security policies
     * should be injected in the page.
     */
    FilterEngine.prototype.getCSPDirectives = function (request) {
        if (!this.config.loadNetworkFilters) {
            return undefined;
        }
        if (request.isSupported !== true || request.isMainFrame() === false) {
            return undefined;
        }
        var matches = this.csp.matchAll(request);
        // No $csp filter found
        if (matches.length === 0) {
            return undefined;
        }
        // Collect all CSP directives and keep track of exceptions
        var disabledCsp = new Set();
        var enabledCsp = new Set();
        for (var i = 0; i < matches.length; i += 1) {
            var filter = matches[i];
            if (filter.isException()) {
                if (filter.csp === undefined) {
                    // All CSP directives are disabled for this site
                    return undefined;
                }
                disabledCsp.add(filter.csp);
            }
            else {
                enabledCsp.add(filter.csp);
            }
        }
        // Combine all CSPs (except the black-listed ones)
        var csps = Array.from(enabledCsp)
            .filter(function (csp) { return !disabledCsp.has(csp); })
            .join('; ') || undefined;
        // Emit event
        if (csps !== undefined) {
            this.emit('csp-injected', csps, request);
        }
        return csps;
    };
    /**
     * Decide if a network request (usually from WebRequest API) should be
     * blocked, redirected or allowed.
     */
    FilterEngine.prototype.match = function (request) {
        var result = {
            exception: undefined,
            filter: undefined,
            match: false,
            redirect: undefined
        };
        if (!this.config.loadNetworkFilters) {
            return result;
        }
        if (request.isSupported) {
            // Check the filters in the following order:
            // 1. $important (not subject to exceptions)
            // 2. redirection ($redirect=resource)
            // 3. normal filters
            // 4. exceptions
            result.filter = this.importants.match(request);
            if (result.filter === undefined) {
                // Check if there is a redirect or a normal match
                result.filter = this.redirects.match(request);
                if (result.filter === undefined) {
                    result.filter = this.filters.match(request);
                }
                // If we found something, check for exceptions
                if (result.filter !== undefined) {
                    result.exception = this.exceptions.match(request);
                }
            }
            // If there is a match
            if (result.filter !== undefined) {
                if (result.filter.isRedirect()) {
                    var redirectResource = this.resources.getResource(result.filter.getRedirect());
                    if (redirectResource !== undefined) {
                        var data = redirectResource.data, contentType = redirectResource.contentType;
                        var dataUrl = void 0;
                        if (contentType.indexOf(';') !== -1) {
                            dataUrl = "data:" + contentType + "," + data;
                        }
                        else {
                            dataUrl = "data:" + contentType + ";base64," + btoaPolyfill(data);
                        }
                        result.redirect = {
                            body: data,
                            contentType: contentType,
                            dataUrl: dataUrl.trim()
                        };
                    } // TODO - else, throw an exception
                }
            }
        }
        result.match = result.exception === undefined && result.filter !== undefined;
        // Emit events if we found a match
        if (result.exception !== undefined) {
            this.emit('request-whitelisted', request, result);
        }
        else if (result.redirect !== undefined) {
            this.emit('request-redirected', request, result);
        }
        else if (result.filter !== undefined) {
            this.emit('request-blocked', request, result);
        }
        return result;
    };
    return FilterEngine;
}(EventEmitter));
export default FilterEngine;
//# sourceMappingURL=engine.js.map